package IkiWiki::Plugin::shortcuts;

use warnings;
use strict;
use Carp;

use IkiWiki '1.00';
use Getopt::Long;
use URI::Escape;
use Data::Dumper;

our $VERSION                = '0.4';
my  %shortcuts              = (
    'deb'       =>  q(http://packages.debian.org/%s),
    'wiki'      =>  q(http://en.wikipedia.org/wiki/%s),
    'google'    =>  q(http://www.google.com/search?q=%s),
    'dbug'      =>  q(http://bugs.debian.org/%s),
    'cpan'      =>  q(http://search.cpan.org/search?mode=dist&query=%s),
);
my  $shortcuts_page         = "shortcuts.mdwn";

sub import { #{{{
    hook(type => "getopt",      id => "shortcuts", call => \&getopt);
    hook(type => "checkconfig", id => "shortcuts", call => \&checkconfig);
    hook(type => "preprocess",  id => "shortcut", call => \&preprocess);
} # }}}

sub getopt () {
    Getopt::Long::Configure('pass_through');
    GetOptions("shortcuts_page" => \$config{shortcuts_page});
}

sub checkconfig () {
    # have we got a shortcuts ikiwiki page ?
    if (defined($config{shortcuts_page})) {
        my $file = srcfile($config{shortcuts_page});

        if (-r $file) {
            my $content = readfile($file);

            # load the shortcuts definitions through a ikiwiki list
            _scan_config_file(\%shortcuts, $content);
        }
        else {
            debug("could not read shortcuts file ${file}");
        }
    }
    else {
        debug("shortcuts page not defined, using defaults");
    }

    return;
}

sub _scan_config_file {
    my  $data_ref   =   shift;
    my  $content    =   shift;

    # parsing shortcuts lists         
    foreach my $line (split(/\n/, $content)) {
        if ($line =~ m{^\s*       # one  more blank spaces
                         \*         # one asterisk
                         \s+        # one  more blank spaces again
                         (\w+)      # one word
                         \s+        # one  more blank spaces  
                         ([^\n]+)   # any char but end of line
                     }xms) {
            my ($name, $url) = ($1, $2);                     
            # is the name uppercase ? 
            if ($name eq uc($name)) {
                _register_shortcut( $name = lc($name) );
            }

            # check special string in url 
            $url .= '%s' if not $url =~ m{[^%]%s}xms;

            # save in global hash                
            $data_ref->{$name} = $url;
        }            
    }

    return;
}

sub _register_shortcut {
    my  $name   =   shift;

    # Register special preprocessor directive
    return hook(type => "preprocess", id => $name, call =>
        _build_preprocessor_code( $name ));
}

sub _build_preprocessor_code {
    my  $name   =   shift;

    return sub {
        return preprocess( $name => '', @_ );
    }        
}

sub preprocess {
    return "" if not @_;
    
    my %args = _extract_ordered_args(@_);

    my  $replaced_text = _transform_shortcut( $args{shortcut}, 
                $args{param_url}, $args{text} );

    if ($replaced_text) {
        # add a dependency 
        add_depends( $args{page}, $shortcuts_page ) if defined($args{page});

        return $replaced_text;
    }
    else {
        return _fail_shortcut( $args{shortcut} );
    }
}

sub _fail_shortcut {
    my  $name       =   shift;

    return sprintf('[[shortcut %s not found]]', $name);
}

sub _transform_shortcut {
    my  $name       =   shift;
    my  $param_url  =   shift;
    my  $text       =   shift;

    #   setting the default text        
    $text = $param_url if not $text;

    if (not exists($shortcuts{$name})) {
        return undef;
    }
    else {
       my $url = _build_url($shortcuts{$name}, $param_url);

       return sprintf('[%s](%s)', $text, $url);
    }
}

sub _extract_ordered_args {
    my  @params     =   @_;
    my  %args = (
        shortcut    =>  undef,
        param_url   =>  undef,
        text        =>  undef,
        page        =>  undef,
        destpage    =>  undef,
    );

    my  $last_params    =   0;      # last param switch
    my  @order          =   qw(shortcut param_url text);  # ordered fields

    #   taking two values each loop
    while (my ($var, $value) = splice(@params, 0, 2)) {
        # is there any added params ? 
        if ($var eq 'page' or $var eq 'destpage') {
            $args{$var}     = $value;
            $last_params    = 1;
        }
        elsif (not $last_params and @order) {
            #   Set the value to the variable name
            $value = $var;
            #   Set the hash key from the next item 
            $var = shift(@order);
            $args{$var} = $value;
        }
    }        
     
    return %args;
}

#
#   build a url with escaped parameters
#

sub _build_url {
    my  $url_text   =   shift;
    my  $param_text =   uri_escape( shift );

    return sprintf($url_text, $param_text );
}

1; # Magic true value required at end of module
__END__

=head1 NAME

Ikiwiki::Plugin::shortcuts - Shortcuts for external links in ikiwiki

=head1 VERSION

This document describes Ikiwiki::Plugin::shortcuts version 0.1

=head1 DESCRIPTION

This ikiwiki plugin search specialized expresions in raw page sources and
replace it with ikiwiki links.

=head1 CONFIGURATION AND ENVIRONMENT

Este mdulo lee un archivo de definiciones llamado shortcuts.conf. 

=head1 DEPENDENCIES

=over 

=item IkiWiki

=item URI::Encode

=back 

=head1 INCOMPATIBILITIES

None reported.

=head1 BUGS AND LIMITATIONS

No bugs have been reported.

Please report any bugs or feature requests to the author.

=head1 AUTHOR

"Vctor Moral"  C<< <victor@taquiones.net> >>

=head1 LICENCE AND COPYRIGHT

Copyright (c) 2006, "Vctor Moral" C<< <victor@taquiones.net> >>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
